@@ -1,4 +1,5 @@
 import datetime
+from pathlib import Path
 
 import pytest
 import pandas as pd
@@ -11,7 +12,7 @@ except ImportError:
     from openpyxl.worksheet import Worksheet as OpenpyxlWorksheet  # noqa: F401
 
 from pdtable import Table
-from pdtable.io.excel import write_excel
+from pdtable.io.excel import write_excel, ExcelWriteBackend
 from pdtable.io._excel_openpyxl import _append_table_to_openpyxl_worksheet, deep_get
 
 
@@ -56,7 +57,8 @@ def test__append_table_to_openpyxl_worksheet():
     assert [ws.cell(r, 4).value for r in range(5, 9)] == [1, 0, 1, 0]
 
 
-def test_write_excel(tmp_path):
+@pytest.mark.parametrize("backend", list(ExcelWriteBackend))
+def test_write_excel(tmp_path, backend):
     # Make a couple of tables
     t = Table(name="foo")
     t["place"] = ["home", "work", "beach", "wonderland"]
@@ -76,7 +78,7 @@ def test_write_excel(tmp_path):
 
     # Write tables to workbook, save, and re-load
     out_path = tmp_path / "foo.xlsx"
-    write_excel([t, t2], out_path)
+    write_excel([t, t2], out_path, backend=backend)
     wb = openpyxl.load_workbook(out_path)
     ws = wb.active
 
@@ -113,7 +115,8 @@ def test_write_excel(tmp_path):
     out_path.unlink()
 
 
-def test_write_excel__multiple_sheets(tmp_path):
+@pytest.mark.parametrize("backend", list(ExcelWriteBackend))
+def test_write_excel__multiple_sheets(tmp_path, backend):
     """write_excel() can write tables to multiple sheets in a workbook"""
 
     # Make a couple of tables
@@ -133,7 +136,7 @@ def test_write_excel__multiple_sheets(tmp_path):
 
     # Write tables to workbook, save, and re-load
     out_path = tmp_path / "foo.xlsx"
-    write_excel({"sheet_one": [t, t2], "sheet_two": t2}, out_path)
+    write_excel({"sheet_one": [t, t2], "sheet_two": t2}, out_path, backend=backend)
     wb = openpyxl.load_workbook(out_path)
 
     # Workbook has the expected sheets
@@ -150,7 +153,8 @@ def test_write_excel__multiple_sheets(tmp_path):
     out_path.unlink()
 
 
-def test_write_excel__style(tmp_path):
+@pytest.mark.parametrize("backend", list(ExcelWriteBackend))
+def test_write_excel__style(tmp_path, backend):
     # Make a couple of tables
     t = Table(name="foo")
     t["place"] = ["home", "work", "beach", "wonderland"]
@@ -176,7 +180,7 @@ def test_write_excel__style(tmp_path):
 
     # Write tables to workbook, save, and re-load
     out_path = tmp_path / "foo.xlsx"
-    write_excel([t, t2, t3], out_path, styles=True)
+    write_excel([t, t2, t3], out_path, styles=True, backend=backend)
     wb = openpyxl.load_workbook(out_path)
     ws = wb.active
 
@@ -189,21 +193,21 @@ def test_write_excel__style(tmp_path):
 
     # Check table formatting
     assert ws["A1"].fill.fill_type == "solid"
-    assert ws["A1"].fill.start_color.value == "00D9D9D9"
-    assert ws["A1"].font.color.value == "001F4E78"
+    assert ws["A1"].fill.start_color.value[2:] == "D9D9D9"  # xlsxwrite prepends hex code with FF
+    assert ws["A1"].font.color.value[2:] == "1F4E78"
     assert ws["A1"].font.bold is True
 
     assert ws["A2"].fill.fill_type == "solid"
-    assert ws["A2"].fill.start_color.value == "00D9D9D9"
-    assert ws["A2"].font.color.value == "00808080"
+    assert ws["A2"].fill.start_color.value[2:] == "D9D9D9"
+    assert ws["A2"].font.color.value[2:] == "808080"
     assert ws["A2"].font.bold is True
 
     assert [ws.cell(3, c).fill.fill_type for c in range(1, 5)] == ["solid"] * 4
-    assert [ws.cell(3, c).fill.start_color.value for c in range(1, 5)] == ["00F2F2F2"] * 4
+    assert [ws.cell(3, c).fill.start_color.value[2:] for c in range(1, 5)] == ["F2F2F2"] * 4
     assert [ws.cell(3, c).font.bold for c in range(1, 5)] == [True] * 4
 
     assert [ws.cell(4, c).fill.fill_type for c in range(1, 5)] == ["solid"] * 4
-    assert [ws.cell(4, c).fill.start_color.value for c in range(1, 5)] == ["00F2F2F2"] * 4
+    assert [ws.cell(4, c).fill.start_color.value[2:] for c in range(1, 5)] == ["F2F2F2"] * 4
     assert [ws.cell(4, c).font.bold for c in range(1, 5)] == [False] * 4
 
     # - table data by column
@@ -230,25 +234,25 @@ def test_write_excel__style(tmp_path):
     # Second table is there as well
     assert ws["A10"].value == "**bar*"
     assert ws["A10"].fill.fill_type == "solid"
-    assert ws["A10"].fill.start_color.value == "00D9D9D9"
-    assert ws["A10"].font.color.value == "001F4E78"
+    assert ws["A10"].fill.start_color.value[2:] == "D9D9D9"
+    assert ws["A10"].font.color.value[2:] == "1F4E78"
     assert ws["A10"].font.bold is True
 
     assert ws["A11"].value == "all"
     assert ws["A11"].fill.fill_type == "solid"
-    assert ws["A11"].fill.start_color.value == "00D9D9D9"
-    assert ws["A11"].font.color.value == "00808080"
+    assert ws["A11"].fill.start_color.value[2:] == "D9D9D9"
+    assert ws["A11"].font.color.value[2:] == "808080"
     assert ws["A11"].font.bold is True
 
     # column headers (transposed)
     assert [ws.cell(r, 1).value for r in range(12, 14)] == ["digit", "spelling"]
     assert [ws.cell(r, 1).fill.fill_type for r in range(12, 14)] == ["solid"] * 2
-    assert [ws.cell(r, 1).fill.start_color.value for r in range(12, 14)] == ["00F2F2F2"] * 2
+    assert [ws.cell(r, 1).fill.start_color.value[2:] for r in range(12, 14)] == ["F2F2F2"] * 2
     assert [ws.cell(r, 1).font.bold for r in range(12, 14)] == [True] * 2
 
     assert [ws.cell(r, 2).value for r in range(12, 14)] == ["-", "text"]
     assert [ws.cell(r, 2).fill.fill_type for r in range(12, 14)] == ["solid"] * 2
-    assert [ws.cell(r, 2).fill.start_color.value for r in range(12, 14)] == ["00F2F2F2"] * 2
+    assert [ws.cell(r, 2).fill.start_color.value[2:] for r in range(12, 14)] == ["F2F2F2"] * 2
     assert [ws.cell(r, 2).font.bold for r in range(12, 14)] == [False] * 2
 
     # column values (transposed)
@@ -263,21 +267,22 @@ def test_write_excel__style(tmp_path):
     # Third table is there as well
     assert ws["A15"].value == "**bas*"
     assert ws["A15"].fill.fill_type == "solid"
-    assert ws["A15"].fill.start_color.value == "00D9D9D9"
-    assert ws["A15"].font.color.value == "001F4E78"
+    assert ws["A15"].fill.start_color.value[2:] == "D9D9D9"
+    assert ws["A15"].font.color.value[2:] == "1F4E78"
     assert ws["A15"].font.bold is True
 
     assert ws["A16"].value == "all"
     assert ws["A16"].fill.fill_type == "solid"
-    assert ws["A16"].fill.start_color.value == "00D9D9D9"
-    assert ws["A16"].font.color.value == "00808080"
+    assert ws["A16"].fill.start_color.value[2:] == "D9D9D9"
+    assert ws["A16"].font.color.value[2:] == "808080"
     assert ws["A16"].font.bold is True
 
     # Teardown
     out_path.unlink()
 
 
-def test_write_excel__custom_style(tmp_path):
+@pytest.mark.parametrize("backend", list(ExcelWriteBackend))
+def test_write_excel__custom_style(tmp_path, backend):
     # Make a table
     t = Table(name="foo")
     t["place"] = ["home", "work", "beach", "wonderland"]
@@ -295,7 +300,7 @@ def test_write_excel__custom_style(tmp_path):
                 "bold": True,
             },
             "fill": {
-                "color": "00AAAAAA",  # leading 'aa' transparency values are accepted (but unused)
+                "color": "AAAAAA",
             },
         },
         "destinations": {
@@ -333,41 +338,41 @@ def test_write_excel__custom_style(tmp_path):
 
     # Write tables to workbook, save, and re-load
     out_path = tmp_path / "foo_custom_style.xlsx"
-    write_excel([t], out_path, styles=style_spec)
+    write_excel([t], out_path, styles=style_spec, backend=backend)
     wb = openpyxl.load_workbook(out_path)
     ws = wb.active
 
     # Check table formatting
     # table name
     assert ws["A1"].fill.fill_type == "solid"
-    assert ws["A1"].fill.start_color.value == "00AAAAAA"
+    assert ws["A1"].fill.start_color.value[2:] == "AAAAAA"
     assert ws["A1"].font.size == 24
-    assert ws["A1"].font.color.value == "00FF0000"
+    assert ws["A1"].font.color.value[2:] == "FF0000"
     assert ws["A1"].font.name == "Times New Roman"
     assert ws["A1"].font.bold is True
 
     # destinations
     assert ws["A2"].fill.fill_type == "solid"
-    assert ws["A2"].fill.start_color.value == "00888888"
-    assert ws["A2"].font.color.value == "000000FF"
+    assert ws["A2"].fill.start_color.value[2:] == "888888"
+    assert ws["A2"].font.color.value[2:] == "0000FF"
     assert ws["A2"].font.bold is False
     assert ws["A2"].font.italic is True
 
     # column names
     assert [ws.cell(3, c).fill.fill_type for c in range(1, nc+1)] == ["solid"] * nc
-    assert [ws.cell(3, c).fill.start_color.value for c in range(1, nc+1)] == ["00777777"] * nc
-    assert [ws.cell(3, c).font.color.value for c in range(1, nc+1)] == ["00444400"] * nc
+    assert [ws.cell(3, c).fill.start_color.value[2:] for c in range(1, nc+1)] == ["777777"] * nc
+    assert [ws.cell(3, c).font.color.value[2:] for c in range(1, nc+1)] == ["444400"] * nc
     assert [ws.cell(3, c).font.bold for c in range(1, nc+1)] == [True] * nc
 
     # column units
     assert [ws.cell(4, c).fill.fill_type for c in range(1, nc+1)] == [None] * nc  # left as default
-    assert [ws.cell(4, c).font.color.value for c in range(1, nc+1)] == ["00440044"] * nc
+    assert [ws.cell(4, c).font.color.value[2:] for c in range(1, nc+1)] == ["440044"] * nc
     assert [ws.cell(4, c).font.bold for c in range(1, nc+1)] == [False] * nc
     assert [ws.cell(4, c).alignment.horizontal for c in range(1, nc+1)] == [None] * nc
 
     # values
     assert [[ws.cell(4 + r, c).fill.fill_type for c in range(1, nc + 1)] for r in range(1, nr + 1)] == [["solid"] * nc] * nr
-    assert [[ws.cell(4 + r, c).fill.start_color.value for c in range(1, nc + 1)] for r in range(1, nr + 1)] == [["00EEEEEE"] * nc] * nr
+    assert [[ws.cell(4 + r, c).fill.start_color.value[2:] for c in range(1, nc + 1)] for r in range(1, nr + 1)] == [["EEEEEE"] * nc] * nr
     assert [[ws.cell(4 + r, c).alignment.horizontal for c in range(1, nc + 1)] for r in range(1, nr + 1)] == [["left"] * nc] * nr
 
 
@@ -380,7 +385,7 @@ def test_write_excel__custom_style(tmp_path):
         ("Invalid.*values", {"values": {"alignment": {"horizontal": "FOOBAR!!!"}}}),
     ]
 )
-def test_write_excel__raises_error_on_invalid_style_spec(tmp_path, err_msg_match, style_spec):
+def test_write_excel_openpyxl_raises_error_on_invalid_style_spec(tmp_path, err_msg_match, style_spec):
     # Make a table
     t = Table(name="foo")
     t["place"] = ["home", "work", "beach", "wonderland"]
@@ -391,7 +396,8 @@ def test_write_excel__raises_error_on_invalid_style_spec(tmp_path, err_msg_match
         write_excel([t], tmp_path / "foo_invalid_style.xlsx", styles=style_spec)
 
 
-def test_write_excel__transposed_table_units_and_values_are_centered_by_default(tmp_path):
+@pytest.mark.parametrize("backend", list(ExcelWriteBackend))
+def test_write_excel__transposed_table_units_and_values_are_centered_by_default(tmp_path, backend):
     # Make a table
     t = Table(name="foo")
     t["place"] = ["home", "work", "beach", "wonderland"]
@@ -403,7 +409,7 @@ def test_write_excel__transposed_table_units_and_values_are_centered_by_default(
     # DEFAULT ALIGNMENT: CENTER
     # Write tables to workbook with default styles, save, and re-load
     out_path = tmp_path / "foo_custom_style.xlsx"
-    write_excel([t], out_path, styles=True)  # <<< Default style
+    write_excel([t], out_path, styles=True, backend=backend)  # <<< Default style
     wb = openpyxl.load_workbook(out_path)
     ws = wb.active
 
@@ -415,7 +421,7 @@ def test_write_excel__transposed_table_units_and_values_are_centered_by_default(
     # Write tables to workbook with custom alignment styles, save, and re-load
     out_path = tmp_path / "foo_custom_style.xlsx"
     left = {"alignment": {"horizontal": "left"}}
-    write_excel([t], out_path, styles={"units": left, "values": left})  # << Custom alignment
+    write_excel([t], out_path, styles={"units": left, "values": left}, backend=backend)  # << Custom alignment
     wb = openpyxl.load_workbook(out_path)
     ws = wb.active
 
@@ -424,7 +430,8 @@ def test_write_excel__transposed_table_units_and_values_are_centered_by_default(
     assert [[ws.cell(2 + c, 2 + r).alignment.horizontal for c in range(1, nc + 1)] for r in range(1, nr + 1)] == [["left"] * nc] * nr
 
 
-def test_write_excel__sep_lines(tmp_path):
+@pytest.mark.parametrize("backend", list(ExcelWriteBackend))
+def test_write_excel__sep_lines(tmp_path, backend):
     # Make a couple of tables
     t = Table(name="foo")
     t["place"] = ["home", "work", "beach", "wonderland"]
@@ -450,7 +457,7 @@ def test_write_excel__sep_lines(tmp_path):
 
     # Write tables to workbook, save, and re-load
     out_path = tmp_path / "foo.xlsx"
-    write_excel([t, t2, t3], out_path, sep_lines=2)
+    write_excel([t, t2, t3], out_path, sep_lines=2, backend=backend)
     wb = openpyxl.load_workbook(out_path)
     ws = wb.active
 
@@ -460,13 +467,28 @@ def test_write_excel__sep_lines(tmp_path):
     assert ws["A17"].value == "**bas*"
 
 
-def test_read_write_excel__round_trip_with_styles(tmp_path):
+@pytest.mark.parametrize("backend", list(ExcelWriteBackend))
+def test_write_excel__empty_table(tmp_path, backend):
+    t = Table(name="foo")
+    t2 = Table(name="bar")
+    t2.metadata.transposed = True
+    out_path = tmp_path / "foo.xlsx"
+    write_excel([t, t2], out_path, sep_lines=2, backend=backend)
+    wb = openpyxl.load_workbook(out_path)
+    ws = wb.active
+    assert ws["A1"].value == "**foo"
+    assert ws["A7"].value == "**bar*"
+
+
+@pytest.mark.parametrize("backend", list(ExcelWriteBackend))
+def test_read_write_excel__round_trip_with_styles(tmp_path, backend):
     """Round-trip reading and writing and re-reading preserves tables"""
     from pdtable import TableBundle, read_excel
-    bundle = TableBundle(read_excel("pdtable/test/io/input/foo.xlsx"))
+    fn_in = Path(__file__).parent / "input" / "foo.xlsx"
+    bundle = TableBundle(read_excel(fn_in))
     out_path = tmp_path / "foo_styled.xlsx"
     # Doesn't crash on write
-    write_excel(bundle, out_path, styles=True)
+    write_excel(bundle, out_path, styles=True, backend=backend)
     # Re-read bundle is same as first one
     bundle2 = TableBundle(read_excel(out_path))
     for t, t2 in zip(bundle, bundle2):
