@@ -66,6 +66,26 @@ def test__table_to_csv__writes_transposed_table():
             ETA;datetime;2020-08-04 08:00:00;2020-08-04 09:00:00;2020-08-04 17:00:00;-
             is_hot;onoff;1;0;1;0
 
+            """
+        )
+
+
+def test__table_to_csv__writes_empty_table():
+    # Make a table with content of various units
+    t = Table(name="empty")
+
+    # Write table to stream
+    with io.StringIO() as out:
+        _table_to_csv(t, out, ";", "-")
+        # Assert stream content is as expected
+        assert out.getvalue() == dedent(
+            """\
+            **empty;
+            all
+
+
+
+
             """
         )
 
@@ -238,3 +258,26 @@ def test_write_csv__with_format_specs():
             
             """
         )
+
+
+def test_write_csv__empty_tables():
+    t1 = Table(name="foo")
+    t2 = Table(name="bar")
+    t2.metadata.transposed = True
+
+    with io.StringIO() as out:
+        write_csv([t1, t2], out)
+        assert out.getvalue() == dedent(
+            """\
+            **foo;
+            all
+            
+            
+            
+            
+            **bar*;
+            all
+            
+            
+            """
+        )
